import { Entity, Multi, Single, System, World, read, listenModify, ModifyEvent, write, ArrMulti } from "pi_ucore/util/ecs";
import { DataC, DataState, LayerType, StateC } from "../component/data";
import { GameState, GlobalState } from "../single/global_state";
import { ActionCmd, ActionName } from "../single/action_cmd";
import { BombList, BombType } from "../single/bomb_list";
import { AnimList, AnimTime, AnimType } from "../single/anim_list";
import { Timeout, Timer } from "pi_scene/scene/single/global";
import { createOneData } from "../utils/init_match";
import { autoSecondSynthesis, secondSynthesis, synthesisBomb } from "../utils/base";
import { AwardType, DataType, SpecialLockType, TypeC } from "../component/type";
import { ExplodeCount } from "../single/explode_count";
import { MoveC, PositionC } from "pi_scene/scene/component/common";
import { DataTypeList } from "../single/data_type_list";
import { RandomBombType } from "../xls/base/base.struct";
import { GoalC } from "../component/goal";
import { cloneObject } from "pi_tools/clone";

/**
 * 游戏操作处理系统
 * 
 * 职责：
 * 1. 处理玩家操作指令（点击、使用道具、交换位置等）
 * 2. 管理炸弹使用和连锁爆炸逻辑
 * 3. 处理物品合成与特殊效果
 * 4. 管理游戏状态转换
 * 5. 协调动画播放与定时器
 */
export class ActionSys implements System {

    // 定义初始化方法（可选）
    setup(_world: World) {
    }

    /**
     * 处理操作指令的核心方法
     * @param e 修改事件对象
     * @param read 只读数据集合 [
     *   0: 全局状态,
     *   1: 实体数据组件,
     *   2: 实体状态组件, 
     *   3: 实体类型组件,
     *   4: 操作指令,
     *   5: 炸弹列表,
     *   6: 动画列表,
     *   7: 定时器,
     *   8: 爆炸计数,
     *   9: 移动组件,
     *   10: 位置组件,
     *   11: 数据类型列表,
     *   12: 目标组件
     * ]
     * @param write 可写数据集合
     */
    @listenModify(ActionCmd)
    @read(GlobalState, [Entity, DataC], [Entity, StateC], [Entity, TypeC], ActionCmd, BombList, AnimList, Timeout, ExplodeCount, [Entity, MoveC]
        , [Entity, PositionC], DataTypeList, [Entity, GoalC])
    listenActionCmd(e: ModifyEvent, read: [Single<GlobalState>, ArrMulti<Entity, DataC>, ArrMulti<Entity, StateC>, ArrMulti<Entity, TypeC>, Single<ActionCmd>
        , Single<BombList>, Single<AnimList>, Single<Timeout>, Single<ExplodeCount>, ArrMulti<Entity, MoveC>, ArrMulti<Entity, PositionC>, Single<DataTypeList>,
        Multi<Entity, GoalC>], write: any) {

        let globalState = read[0].data;
        let dataC = read[1];
        let stateC = read[2];
        let typeC = read[3];
        let action = read[4].data;
        let bombList = read[5].data;
        let animList = read[6].data;
        let timeout = read[7].data;
        let explode = read[8].data;
        let moveC = read[9];
        let positionC = read[10];
        let dataTypeList = read[11].data;
        let goalC = read[12];

        const indexs = globalState.areaIndexs.get(globalState.curArea);
        const [w, h] = globalState.areaMap.get(globalState.curArea);
        if (action.cmd === ActionName.change) {
            // TODO
        }
        else if (action.cmd === ActionName.click) {// 点击
            let [x, y] = action.pos;
            let id = indexs[LayerType.Lock][x][y];
            // 锁层有物品不能点击
            if (id) return;

            id = indexs[LayerType.Data][x][y];
            if (id) {
                // 非正常状态不能点击
                if (stateC.get(id).value !== DataState.Normal) return;
                let type = typeC.get(id);
                console.log('点击', id, type.value, DataType[type.value], type.subType)
                // 点击合成炸弹
                if (type.value === DataType.SynthesisBomb) {
                    if (globalState.canOperate()) {
                        stateC.get(id).value = DataState.WaitExplode;
                        globalState.gameState = GameState.Waiting;
                    }
                    return;
                }
                // 点击奖励物品
                else if (type.value === DataType.AwardData) {
                    stateC.get(id).value = DataState.Exploded;
                    if (typeC.get(id).subType === AwardType.BombBag) { // 炸弹补充
                        bombList.list.unshift(BombType.SingleBomb);
                        bombList.list = bombList.list;
                    }
                    return
                }
                else {
                    // 删除没用的指令
                    action.list.pop()
                }
            }
        }
        else if (action.cmd === ActionName.useBomb) {// 使用雷
            if (!globalState.canOperate()) return;
            let list = JSON.parse(action.arg);
            let { posList, typeList, idList, time, bombs, realPoscount, cannotBombList } = this.bombed(indexs, list, w, h, bombList, animList, typeC, stateC)
            // 没有有效位置 不能爆炸
            if (posList.length === 0) {
                let { posList, idList, dataType } = this.synthesis(indexs, list, w, h, typeC)
                console.log('合成物品', posList, idList, dataType)
                // 有效位置不足4个 不能合成
                if (posList.length !== 4) return;

                let goal = goalC.get(idList[0]);
                goal.idList = idList;
                goal.dataCount += 1;
                return;
            }
            if (bombs.length > 0) {
                bombList.waitBombs.push(...bombs.map(v => v[0]));
                bombList.waitBombs = bombList.waitBombs;
            }
            globalState.cannotBombList = cannotBombList;
            this.afterBomb(write, indexs, posList, typeList, idList, time, bombs, stateC, typeC, timeout, animList, globalState, bombList, goalC);
            // 通知爆炸格子数
            timeout.value = new Timer(AnimTime.BombWire, () => {
                if (explode.oldValue && !explode.isEnd) explode.isEnd = false;
                else explode.isEnd = bombList.waitBombs.length === 0;
                explode.value = realPoscount;
            })
        }
        else if (action.cmd === ActionName.swap) {// 交换
            if (!globalState.canOperate()) return;

            let list = JSON.parse(action.arg);
            if (!list[0] || !list[1]) return
            const dataIndexs = indexs[LayerType.Data];
            let startId = dataIndexs[list[0][0]][list[0][1]];
            let endId = dataIndexs[list[1][0]][list[1][1]];
            let startLock = indexs[LayerType.Lock][list[0][0]][list[0][1]];
            let endLock = indexs[LayerType.Lock][list[1][0]][list[1][1]];
            if (!startId || startLock || typeC.get(startId).value > DataType.EmptyBox) return
            if (!endId || endLock || typeC.get(endId).value > DataType.EmptyBox) return

            // console.log('指令参数', list, '随机种子', globalState.seed, bombList.list);
            const startPos = JSON.parse(JSON.stringify(dataC.get(startId).pos));
            const endPos = JSON.parse(JSON.stringify(dataC.get(endId).pos));

            animList.addAnimData(0, {
                type: AnimType.Swap,
                time: AnimTime.Swap,
                posList: [endPos],
                startPos: [startPos],
            })
            timeout.value = new Timer(AnimTime.Swap, () => {
                bombList.list.shift()
                bombList.list = bombList.list;

                this.swapData(dataC, moveC, positionC, dataIndexs, endPos, startId, true)
                this.swapData(dataC, moveC, positionC, dataIndexs, startPos, endId, true)
            })
        }
        else if (action.cmd === ActionName.shuffle) {// 混乱
            if (!globalState.canOperate()) return;
            // console.log('随机种子', globalState.seed);

            const locArr: number[][] = [];
            const idArr: number[] = [];
            const dataIndexs = indexs[LayerType.Data];
            dataIndexs.forEach((v, k1) => {
                v.forEach((id, k2) => {
                    const lock = indexs[LayerType.Lock][k1][k2];
                    if (!id || lock || typeC.get(id).value > DataType.EmptyBox) return
                    idArr.push(id);
                    locArr.push([k1, k2]);
                })
            })
            if (idArr.length <= 0) return
            animList.addAnimData(0, {
                type: AnimType.Shuffle,
                time: AnimTime.Shuffle,
                posList: locArr,
            })
            timeout.value = new Timer(AnimTime.Shuffle, () => {
                dataTypeList.rand.shuffle(locArr);
                idArr.forEach((id, k) => this.swapData(dataC, moveC, positionC, dataIndexs, locArr[k], id, true));
            })
        }
        else if (action.cmd === ActionName.addSynthesisBomb) {// 添加合成雷

            let arg = JSON.parse(action.arg);
            const fixBooms = arg.fixBooms || [];
            const startPos = arg.startPos || [];
            console.log('添加合成雷', arg);

            const idArr: number[] = [];
            const locArr: Map<number, number[]> = new Map();
            const dataIndexs = indexs[LayerType.Data];
            const randomCount = fixBooms.length || arg.count;
            dataIndexs.forEach((v, k1) => {
                v.forEach((id, k2) => {
                    const lock = indexs[LayerType.Lock][k1][k2];
                    if (!id || lock || typeC.get(id).value > DataType.EmptyBox) return
                    idArr.push(id);
                    locArr.set(id, [k1, k2]);
                })
            })
            if (idArr.length <= 0) return
            dataTypeList.rand.shuffle(idArr);
            if (idArr.length > randomCount) idArr.length = randomCount;
            idArr.forEach((id, i) => {
                this.addSynthesisBomb(write, dataIndexs, locArr.get(id), globalState, dataTypeList, timeout, animList, fixBooms[i], startPos, typeC.get(id), stateC.get(id))
            });
        }

        console.warn('指令数组', action.list, '随机种子', globalState.seed);
    }

    @listenModify(GlobalState)
    @read(GlobalState, BombList, [Entity, StateC], [Entity, DataC], [Entity, TypeC], AnimList, Timeout)
    listenGlobalState(e: ModifyEvent, read: [Single<GlobalState>, Single<BombList>, ArrMulti<Entity, StateC>, ArrMulti<Entity, DataC>, ArrMulti<Entity, TypeC>, Single<AnimList>, Single<Timeout>], write: any) {
        let globalState = read[0].data;
        let bombList = read[1].data;
        let stateC = read[2];
        let dataC = read[3];
        let typeC = read[4];
        let animList = read[5].data;
        let timeout = read[6].data;
        if (globalState.gameState !== GameState.DropEnd) return;

        if (bombList.waitBombs.length !== 0) {
            // 依次引爆
            let id = bombList.waitBombs.shift();
            bombList.waitBombs = bombList.waitBombs;
            stateC.get(id).value = DataState.WaitExplode;
            globalState.gameState = GameState.Waiting;

        }
        if (bombList.waitSynthesis.length !== 0) {
            // TODO自动合成二级雷
            if (globalState.mapCfg.autoSecondSynthesis) {
                for (let i = 0; i < bombList.waitSynthesis.length; i++) {
                    let { centerPos, bombType, bombShapePos } = autoSecondSynthesis(i, bombList.waitSynthesis, globalState, dataC, typeC);
                    if (bombType) {
                        // TODO 合成炸弹动画
                        // 替换为新的炸弹类型
                        animList.addAnimData(bombList.waitSynthesis[i], {
                            type: AnimType.AutoSynthesisBomb,
                            time: AnimTime.AutoSynthesisBomb,
                            arg: bombType,
                            posList: [centerPos],
                            startPos: bombShapePos,
                        })
                        typeC.get(bombList.waitSynthesis[i]).subType = bombType;
                        timeout.value = new Timer(AnimTime.AutoSynthesisBomb, () => {
                            globalState.gameState = GameState.DropEnd;
                        })
                    }
                }
            }
            bombList.waitSynthesis = [];
        }
    }

    /**
     * 处理炸弹放置逻辑
     * @param indexs 三维索引数组 [层类型][x坐标][y坐标]
     * @param list 炸弹作用位置列表
     * @param w 当前区域宽度
     * @param h 当前区域高度
     * @param bombList 炸弹列表数据
     * @param animList 动画列表数据
     * @param typeC 类型组件
     * @param stateC 状态组件
     * @returns 爆炸相关信息 {
     *   posList: 有效爆炸位置,
     *   typeList: 类型列表,
     *   idList: 实体ID列表,
     *   time: 总动画时间,
     *   bombs: 延迟引爆的炸弹,
     *   realPoscount: 实际作用格子数,
     *   cannotBombList: 不可炸列表
     * }
     */
    bombed(indexs: number[][][], list: number[][], w: number, h: number, bombList: BombList, animList: AnimList, typeC, stateC) {
        let bombType = bombList.list[0]
        let posList = [], idList = [], typeList = [], realPoscount = 0, cannotBombList = [];
        for (let [x, y] of list) {
            if (x < 0 || x >= w || y < 0 || y >= h) continue;
            let id = indexs[LayerType.Lock][x][y];
            // 空地不算格子
            if (id && typeC.get(id).isEmptyNoBoom()) return { posList: [] };
            if (!id) id = indexs[LayerType.Data][x][y];
            // 草垛可以被炸 但不能合成炸弹
            if (!id) id = indexs[LayerType.Lawn][x][y];
            // 包含空位置不能炸
            if (id <= 0) return { posList: [] };

            realPoscount++;
            // ID已包含
            if (idList.includes(id)) continue;
            // 不能炸的类型
            if (typeC.get(id).cannotBomb) {
                cannotBombList.push(id);
                continue;
            }
            posList.push([x, y])
            idList.push(id);
            typeList.push(typeC.get(id));
        }
        // 没有有效位置 不消耗炸弹
        if (!posList.length) return { posList }

        let bombs = [];
        for (let i = 0; i < posList.length; i++) {
            let id = idList[i];
            let type = typeList[i];
            // 合成炸弹 延迟引爆
            if (type.value === DataType.SynthesisBomb) {
                bombs.push([id, type, posList[i]]);
            } else {
                stateC.get(id).value = DataState.WaitExplode;
            }
        }

        // 消耗炸弹
        bombList.times++;
        bombList.list.shift()
        bombList.list = bombList.list;
        // 引燃导线
        animList.addAnimData(0, {
            type: AnimType.BombWire,
            time: AnimTime.BombWire,
            arg: bombType,
            posList
        })
        // console.log("----Anim----AnimType.Exploding----1111111---", posList, bombs)
        // 爆炸动画
        // 预处理本次爆炸是否触发联动
        let info = synthesisBomb(posList, typeList, false);
        animList.addAnimData(0, {
            type: AnimType.Exploding,
            time: AnimTime.Exploding,
            arg: bombType,
            posList,
            isAnnex: info.bombTypes && info.bombTypes.length > 0
        })
        // 爆炸动画
        animList.addAnimData(0, {
            type: AnimType.ExplodingAni,
            time: AnimTime.ExplodingAni,
            posList
        })
        // 气浪
        animList.addAnimData(0, {
            type: AnimType.AirWave,
            time: AnimTime.AirWave,
            posList
        })
        let time = AnimTime.BombWire + AnimTime.Exploding + AnimTime.AirWave
        return { posList, typeList, idList, time, bombs, realPoscount, cannotBombList }
    }

    /**
     * 合成特殊物品逻辑
     * @param indexs 三维索引数组 [层类型][x坐标][y坐标]
     * @param list 待合成位置列表
     * @param w 当前区域宽度
     * @param h 当前区域高度
     * @param typeC 类型组件
     * @returns 合成信息 {
     *   posList: 有效合成位置,
     *   idList: 实体ID列表,
     *   dataType: 合成物品类型
     * }
     */
    synthesis(indexs: number[][][], list: number[][], w: number, h: number, typeC) {
        let posList = [], idList = [], dataType = null;
        for (let [x, y] of list) {
            if (x < 0 || x >= w || y < 0 || y >= h) continue;
            let id = indexs[LayerType.Lock][x][y];
            // 空地不算格子
            if (id && typeC.get(id).isEmptyNoBoom()) return { posList: [] };
            if (!id) id = indexs[LayerType.Data][x][y];
            // 包含空位置不能炸
            if (!id) return { posList: [] };

            // 合成物品
            let type = typeC.get(id);
            if (type.value !== DataType.SynthesisData || idList.includes(id)) return { posList: [] }
            // 不同的合成物品 不能合成
            if (dataType && dataType !== type.subType) return { posList: [] }
            posList.push([x, y])
            idList.push(id);
            dataType = type.subType;
        }
        return { posList, idList, dataType }
    }

    // 爆炸后
    afterBomb(world: World, indexs: number[][][], posList: [number, number][], typeList: TypeC[], idList: number[], time: number, bombs: [number, TypeC, [number, number]][], stateC, typeC, timeout: Timeout, animList: AnimList, globalState: GlobalState, bombList: BombList, goalC) {
        // 合成炸弹
        let { bombTypes, pos, dataType, superBombType } = synthesisBomb(posList, typeList);
        timeout.value = new Timer(time, () => {
            let bombFg = false;  // 是否有格子爆炸
            for (let i = 0; i < idList.length; i++) {
                let id = idList[i];
                let type = typeC.get(id)
                if (!type) continue;
                if (stateC.get(id).value === DataState.WaitExplode) {
                    type.needBombTimes--;
                    stateC.get(id).value = DataState.Exploded;
                    bombFg = true;
                }
            }

            if (!bombTypes || bombTypes.length <= 0) return; // 不能合成炸弹
            // 待合成炸弹的位置 阻挡草垛弹出物品
            bombList.waitSynthesisPos = cloneObject(pos);

            for (let i = 0; i < bombTypes.length; i++) {
                const curPos = pos[i];
                let id = indexs[LayerType.Data][curPos[0]][curPos[1]];
                // 合成炸弹动画
                animList.addAnimData(id, {
                    type: AnimType.SynthesisBomb,
                    time: AnimTime.SynthesisBomb,
                    arg: bombTypes[i],
                    posList: [curPos],
                    startPos: posList,
                    dataType
                })
            }
            timeout.value = new Timer(AnimTime.SynthesisBomb, () => {
                for (let i = 0; i < bombTypes.length; i++) {
                    const curPos = pos[i];
                    const bombType = bombTypes[i];
                    let newId = createOneData(world, curPos, globalState.curArea, LayerType.Data, [1, 1], 1, DataType.SynthesisBomb, bombType);
                    // 销毁原位置上的物品
                    let id = indexs[LayerType.Data][curPos[0]][curPos[1]];
                    if (id) stateC.get(id).value = DataState.Exploded;
                    indexs[LayerType.Data][curPos[0]][curPos[1]] = newId;
                    console.log('合成一级炸弹', newId, curPos, BombType[bombType]);
                    bombList.waitSynthesis.push(newId);

                    // 引爆合成的雷
                    bombList.waitBombs.push(newId);
                }
                bombList.waitBombs = bombList.waitBombs;
            })

            // 全是合成雷，没有格子爆炸 则手动修改状态，触发合成雷爆炸
            if (!bombFg) {
                timeout.value = new Timer(AnimTime.SynthesisBomb, () => {
                    globalState.gameState = GameState.DropEnd;
                    bombList.superBombType = superBombType;
                })
            }
        })
    }


    // 交换数据
    swapData(dataC: ArrMulti<Entity, DataC>, moveC: ArrMulti<Entity, MoveC>, positionC: ArrMulti<Entity, PositionC>, dataIndexs: number[][], endPos: number[], id: number, useMove: boolean) {

        if (useMove) {
            moveC.get(id).add([endPos[0], endPos[1], 0], 80);
        } else {
            positionC.get(id).value = [endPos[0], endPos[1], 0]
        }
        dataIndexs[endPos[0]][endPos[1]] = id;
        dataC.get(id).pos = [endPos[0], endPos[1]];
    }

    private addSynthesisBomb(world: World, dataIndexs: number[][], endPos: number[], globalState: GlobalState, dataTypeList: DataTypeList
        , timeout: Timeout, animList: AnimList, fixBoom: number, startPos: number[], type: TypeC, state: StateC) {
        if (!fixBoom) {
            const randomBombType = JSON.parse(RandomBombType)
            const index = dataTypeList.rand.nextInt(0, randomBombType.length - 1);
            fixBoom = randomBombType[index];
        }

        // 合成炸弹动画
        animList.addAnimData(0, {
            type: AnimType.AutoAddBomb,
            time: AnimTime.AutoAddBomb,
            arg: fixBoom,
            posList: [endPos],
            startPos: [startPos]
        })
        timeout.value = new Timer(AnimTime.AutoAddBomb, () => {
            type.needBombTimes--;
            state.value = DataState.Exploded;

            let newId = createOneData(world, [endPos[0], endPos[1]], globalState.curArea, LayerType.Data, [1, 1], 1, DataType.SynthesisBomb, fixBoom);
            dataIndexs[endPos[0]][endPos[1]] = newId;
        })

    }
}