import { Entity, Multi, Single, System, World, read, listenModify, ModifyEvent, write, ArrMulti } from "pi_ucore/util/ecs";
import { DataC, DataState, LayerType, SizeC, StateC } from "../component/data";
import { GlobalState } from "../single/global_state";
import { AroundPosEight, AroundPosFour } from "../utils/base";
import { DataType, LockTypeC, TypeC } from "../component/type";
import { AnimList, AnimTime, AnimType } from "../single/anim_list";
import { Timeout, Timer } from "pi_scene/scene/single/global";
import { GoalC, GoalType } from "../component/goal";
import { ExplodeCount } from "../single/explode_count";
/**
 * 周围物品爆炸影响系统
 * 
 * 职责：
 * 1. 处理爆炸物对周围格子的连锁影响
 * 2. 管理被炸实体的状态转换（正常 -> 周围爆炸中 -> 已爆炸）
 * 3. 处理特殊目标（如美人鱼锁）的收集逻辑
 * 4. 协调爆炸动画和时间控制
 * 
 * 主要功能：
 * - 当实体被标记为WaitExplode状态时，记录其影响范围
 * - 根据爆炸类型（火箭/普通）计算影响范围（8方向/4方向）
 * - 更新被影响实体的爆炸次数需求
 * - 处理目标计数和特殊动画触发
 */
export class AroundSys implements System {
    dirtyList: number[][] = []; // 被炸实体记录列表 [实体ID, x坐标, y坐标, 子类型?]

    /**
     * 系统初始化方法
     * @param _world ECS世界对象
     */
    setup(_world: World) {
    }

    /**
     * 主运行逻辑
     * @param read 只读组件集合：
     *   [0] GlobalState - 全局状态
     *   [1] DataC - 实体数据组件
     *   [2] StateC - 实体状态组件  
     *   [3] TypeC - 实体类型组件
     *   [4] LockTypeC - 锁定类型组件
     *   [5] Timeout - 超时控制
     *   [6] GoalC - 目标组件
     *   [7] AnimList - 动画列表
     *   [8] ExplodeCount - 爆炸计数
     * @param write 可写组件（未使用）
     */
    @read(GlobalState, [Entity, DataC], [Entity, StateC], [Entity, TypeC], [Entity, LockTypeC], Timeout, [Entity, GoalC], AnimList, ExplodeCount)
    run(read: [
        Single<GlobalState>, 
        ArrMulti<Entity, DataC>, 
        ArrMulti<Entity, StateC>, 
        ArrMulti<Entity, TypeC>, 
        ArrMulti<Entity, LockTypeC>,
        Single<Timeout>, 
        Multi<Entity, GoalC>, 
        Single<AnimList>, 
        Single<ExplodeCount>
    ], write: any) {
        // 无待处理爆炸时直接返回
        if (this.dirtyList.length === 0) return;

        // 初始化组件引用
        let globalState = read[0].data;        // 全局游戏状态
        let dataC = read[1];                   // 实体数据组件
        let stateC = read[2];                  // 实体状态组件
        let typeC = read[3];                   // 实体类型组件 
        let lockTypeC = read[4];               // 锁定类型组件
        let timeout = read[5].data;            // 超时控制器
        let goalC = read[6];                   // 目标组件
        let animList = read[7].data;           // 动画列表
        let explode = read[8].data;            // 爆炸计数
        
        // 获取当前区域索引和地图尺寸
        let indexs = globalState.areaIndexs.get(globalState.curArea);
        let [w, h] = globalState.areaMap.get(globalState.curArea);

        console.log("AroundSys dirtyList = ", this.dirtyList);
        let idList = new Map(); // 记录实体被炸次数 [实体ID, 被炸次数]
        
        // 处理每个被炸位置
        for (let [id, x, y] of this.dirtyList) {
            // 检查是否是火箭爆炸的终点位置
            const rocketIndex = explode.rocketEndPos.findIndex(v => v[0] === x && v[1] === y);
            const isBoomByRocket = rocketIndex >= 0;  // 是否是火箭引发的爆炸
            if (isBoomByRocket) {
                explode.rocketEndPos.splice(rocketIndex, 1);  // 移除已处理的火箭终点
            }

            if (!typeC.get(id)) continue;  // 跳过无类型组件的实体
            
            let data = dataC.get(id);
            // 仅处理需要1次爆炸且位于数据层的实体
            if (typeC.get(id).needBombTimes !== 1 || data.layer !== LayerType.Data) continue;

            // 根据爆炸类型确定影响范围
            const AroundPosList = isBoomByRocket ? AroundPosEight : AroundPosFour;
            
            // 遍历周围位置
            for (let [i, j] of AroundPosList) {
                let x1 = x + i, y1 = y + j;
                
                // 边界检查
                if (x1 < 0 || x1 >= w || y1 < 0 || y1 >= h) continue;
                
                // 获取锁定层实体ID
                let id1 = indexs[LayerType.Lock][x1][y1];
                if (!id1) continue;
                
                // 检查是否允许被周围爆炸影响
                if (!lockTypeC.get(id1)?.aroundBombEffect) continue;
                
                // 处理多次记录限制
                if (!lockTypeC.get(id1).recordMultipleTimes) {
                    // 如果实体已在处理列表或当前正在爆炸，则跳过
                    if ((this.dirtyList.find(v => v[0] === id1) || idList.get(id1))) continue;
                    if (stateC.get(id1).isExploding()) continue;
                }
                
                // 更新被炸次数
                let count = idList.get(id1) || 0;
                if (!stateC.get(id1).isExploding()) {
                    stateC.get(id1).value = DataState.AroundExplode;  // 标记为周围爆炸状态
                }
                idList.set(id1, ++count);
                console.log(id, [x, y], '炸周围', id1, count, [x1, y1]);
            }

        }
        console.log("AroundSys idList = ", idList);
        
        // 计算动画时间
        let time = AnimTime.Exploding;
        // 如果当前有合成炸弹或飞行动画，需要累加时间
        if (animList.cur && [AnimType.SynthesisBomb, AnimType.Flying].includes(animList.cur.type)) {
            time += animList.cur.time;
        }

        // 设置超时处理实际爆炸效果
        timeout.value = new Timer(time, () => {
            for (let [id, count] of idList) {
                let state = stateC.get(id);
                let type = typeC.get(id);
                
                // 更新需要爆炸次数
                if (type) {
                    type.needBombTimes -= count;
                    type.needBombTimes = Math.max(type.needBombTimes, 0);  // 确保不小于0
                }
                
                // 处理目标组件
                if (goalC.get(id)?.type === GoalType.OutPut) {
                    let goal = goalC.get(id);
                    let data = dataC.get(id);
                    
                    if (goal.dataType === DataType.MermaidLock) {
                        // 美人鱼锁特殊处理
                        if (goal.dataCount < goal.goalCount && goal.dataCount + count >= goal.goalCount) {
                            // 触发解锁动画
                            animList.addAnimData(id, {
                                type: AnimType.UnlockMermaid,
                                time: AnimTime.UnlockMermaid,
                                posList: [data.pos]
                            });
                            state.value = DataState.WaitExplode;
                            
                            // 设置解锁完成后的回调
                            timeout.value = new Timer(AnimTime.UnlockMermaid, (id) => {
                                goal.dataCount += count;
                                state.value = DataState.Exploded;
                                console.warn('美人鱼收集完成', id, data.pos);
                            }, id);
                            return;
                        } else {
                            goal.dataCount += count;  // 普通计数增加
                        }
                    } else {
                        goal.dataCount += count;
                        console.log(id, '炸周围111', goal.dataCount, type.needBombTimes, DataState[state.value]);
                    }
                }
                
                // 状态转换处理
                if (state?.value === DataState.AroundExplode) {
                    state.value = DataState.Exploded;  // 周围爆炸完成
                }
                
                // 处理同时被直接炸和周围影响的情况
                if (state?.value === DataState.Exploded && 
                    !type.needBombTimes && 
                    state?.oldValue === DataState.WaitExplode) {
                    state.value = DataState.Exploded;  // 强制状态更新通知显示层
                }
            }
        });
        this.dirtyList = [];
    }

    /**
     * 监听状态变化事件
     * @param e 修改事件
     * @param read 只读组件：
     *   [0] StateC - 状态组件
     *   [1] DataC - 数据组件
     *   [2] SizeC - 尺寸组件
     *   [3] TypeC - 类型组件
     * @param write 可写组件（未使用）
     */
    @listenModify([Entity, StateC])
    @read([Entity, StateC], [Entity, DataC], [Entity, SizeC], [Entity, TypeC])
    listenState(e: ModifyEvent, read: [ArrMulti<Entity, StateC>, ArrMulti<Entity, DataC>, Multi<Entity, SizeC>, ArrMulti<Entity, TypeC>], write: any) {
        let stateC = read[0];
        let dataC = read[1];
        let sizeC = read[2];
        let typeC = read[3];
        // 冰块鱼收集不能影响周围
        if (stateC.get(e.id).value === DataState.WaitExplode && typeC.get(e.id).value !== DataType.Donut) {
            let data = dataC.get(e.id);
            let size = sizeC.get(e.id);
            let [x, y] = data.pos;
            for (let i = 0; i < size.width; i++) {
                for (let j = 0; j < size.height; j++) {
                    this.dirtyList.push([e.id, x + i, y + j, typeC.get(e.id).subType]);
                }
            }
        }
    }
}
