import { Entity, Multi, Single, System, World, read, listenModify, ModifyEvent, write, ArrMulti } from "pi_ucore/util/ecs";
import { DataC, DataState, LayerType, SizeC, StateC } from "../component/data";
import { GameState, GlobalState } from "../single/global_state";
import { AroundPosEight, randomPos } from "../utils/base";
import { AnimData, AnimList, AnimTime, AnimType } from "../single/anim_list";
import { FrameTime, Timeout, Timer } from "pi_scene/scene/single/global";
import { BombList, BombType } from "../single/bomb_list";
import { getCfg } from "pi_common_tools/util/db_tool";
import { BombTypeCfg } from "../xls/bomb/bomb.struct";
import { AwardType, DataType, EventDataType, LockTypeC, NormalLockType, SpecialLockType, TypeC } from "../component/type";
import { ExplodeCount } from "../single/explode_count";
import { EventDataCfg } from "../xls/data_type/event.struct";
import { Rand } from "pi_math/rand";
import { Calc } from "pi_common_tools/util/calc";
import { Collect } from "../single/collect";
import { BombProgress, FirstBombData } from "../single/bomb_progress";
import { TimeStep } from "../single/time_step";
import { GoalC } from "../component/goal";
/**
 * 炸弹系统 - 处理合成炸弹的引爆逻辑和爆炸效果
 * 
 * 主要职责：
 * 1. 监听炸弹实体状态变化，触发爆炸流程
 * 2. 处理不同类型的炸弹效果（横雷、竖雷、全屏爆炸等）
 * 3. 管理爆炸动画和特效时序
 * 4. 处理多阶段爆炸逻辑（如火箭二次爆炸）
 * 5. 维护爆炸过程中的状态数据（如飞机目标位置）
 * 
 * 系统依赖：
 * - 全局状态(GlobalState)
 * - 动画列表(AnimList)
 * - 炸弹列表(BombList)
 * - 爆炸计数器(ExplodeCount)
 */
export class BombSys implements System {
    /** 
     * 飞机炸弹选择的目标位置列表 
     * 用于避免同时引爆多个飞机雷时选择相同目标
     */
    airplanePosList: number[][] = [];

    /** 系统初始化方法（ECS生命周期方法） */
    setup(_world: World) {
    }

    /**
     * 监听实体状态变化 - 当炸弹进入等待爆炸状态时触发
     * @param e 修改事件对象
     * @param read 只读数据集合 [
     *   GlobalState,      // 全局游戏状态
     *   DataC[],         // 实体数据组件
     *   StateC[],        // 实体状态组件
     *   TypeC[],         // 实体类型组件
     *   LockTypeC[],     // 锁定类型组件
     *   SizeC[],         // 实体尺寸组件
     *   BombProgress     // 炸弹进度数据
     * ]
     * @param write 可写数据集合 [
     *   AnimList,        // 动画列表
     *   Timeout,         // 定时器
     *   Calc,            // 计算工具
     *   ExplodeCount,    // 爆炸计数器
     *   BombList,        // 炸弹列表
     *   FrameTime        // 帧时间
     * ]
     */
    @listenModify([Entity, StateC])
    @read(GlobalState, [Entity, DataC], [Entity, StateC], [Entity, TypeC], [Entity, LockTypeC]
        , [Entity, SizeC], BombProgress, [Entity, GoalC], TimeStep)
    @write(AnimList, Timeout, Calc, ExplodeCount, BombList, FrameTime)
    listenState(e: ModifyEvent, read: [Single<GlobalState>, ArrMulti<Entity, DataC>, ArrMulti<Entity, StateC>, ArrMulti<Entity, TypeC>, ArrMulti<Entity, LockTypeC>
        , ArrMulti<Entity, SizeC>, Single<BombProgress>, ArrMulti<Entity, GoalC>, Single<TimeStep>],
        write: [Single<AnimList>, Single<Timeout>, Single<Calc>, Single<ExplodeCount>, Single<BombList>, Single<FrameTime>]) {
        let globalState = read[0].data;
        let dataC = read[1];
        let stateC = read[2];
        let typeC = read[3];
        let lockTypeC = read[4];
        let sizeC = read[5];
        let progress = read[6].data;
        let goalC = read[7];
        let timeStep = read[8].data;

        let state = stateC.get(e.id);
        let type = typeC.get(e.id);
        let animList = write[0].data;
        let timeout = write[1].data;
        let calc = write[2].data;
        let explode = write[3].data;
        let bombList = write[4].data;
        let frame = write[5].data;
        if (state.value !== DataState.WaitExplode || type.value !== DataType.SynthesisBomb) return;
        let bombType = type.subType as BombType;
        let cfg: BombTypeCfg = getCfg(BombTypeCfg, bombType);
        if (!cfg) throw new Error('错误的炸弹类型，找不到配置表: ' + bombType);
        // 延迟引爆 格子爆炸时引爆
        timeout.value = new Timer(AnimTime.Exploding, () => {
            console.warn('延迟引爆', e.id)
            this.dealBomb(e.id, bombType, cfg, globalState, timeout, animList, bombList, explode, frame, lockTypeC, typeC, stateC, dataC, sizeC, progress, goalC, timeStep)
        })
    }

    @listenModify(BombProgress)
    @read(BombProgress, BombList, [Entity, StateC], [Entity, DataC], [Entity, TypeC], AnimList, Timeout, Calc, Collect, ExplodeCount, GlobalState
        , [Entity, LockTypeC], [Entity, SizeC], FrameTime, [Entity, GoalC], TimeStep)
    listenGlobalState(e: ModifyEvent, read: [Single<BombProgress>, Single<BombList>, ArrMulti<Entity, StateC>, ArrMulti<Entity, DataC>, ArrMulti<Entity, TypeC>
        , Single<AnimList>, Single<Timeout>, Single<Calc>, Single<Collect>, Single<ExplodeCount>, Single<GlobalState>, ArrMulti<Entity, LockTypeC>
        , ArrMulti<Entity, SizeC>, Single<FrameTime>, ArrMulti<Entity, GoalC>, Single<TimeStep>], write: any) {
        let progress = read[0].data;
        let bombList = read[1].data;
        let stateC = read[2];
        let dataC = read[3];
        let typeC = read[4];
        let animList = read[5].data;
        let timeout = read[6].data;
        let calc = read[7].data;
        let collect = read[8].data;
        let explode = read[9].data;
        let globalState = read[10].data;
        let lockTypeC = read[11];
        let sizeC = read[12];
        let frame = read[13].data;
        let goalC = read[14];
        let timeStep: TimeStep = read[15].data;
        if (!progress.isExploded || !progress.map.get(progress.curEntity)) return;

        let [w, h] = globalState.areaMap.get(globalState.curArea);
        let indexs = globalState.areaIndexs.get(globalState.curArea);

        // 延迟一帧计算第二段爆炸 等xidexs更新完成
        let { lastBomb, lastBombCfg, lastList, realPoscount, otherTime } = progress.map.get(progress.curEntity);
        timeout.value = new Timer(frame.frameInterval + otherTime, () => {
            let [x, y, id] = lastBomb
            let cfg = lastBombCfg;
            const limitPos = this.getPosFromNewTimeAwardData(timeStep, goalC, dataC)
            let { secondList, rocketEndPos, otherEndPos, realPoscount: secondRealPoscount } =
                this.secondPart(lastList.concat(limitPos), indexs, cfg, w, h, [x, y], calc.rand, typeC, dataC, lockTypeC, collect, bombList)
            if (secondList.length) {
                this.readyBomb(id, secondList, w, h, indexs, cfg, animList, timeout, bombList, globalState, stateC, typeC, dataC, goalC, timeStep, true, [x, y], rocketEndPos, otherEndPos, sizeC);
                explode.setRocketEndPos(rocketEndPos);
                // 通知爆炸格子数
                timeout.value = new Timer(cfg.flyTime, () => {
                    explode.isEnd = bombList.waitBombs.length === 0;
                    explode.value = realPoscount + secondRealPoscount;
                })
            }
            progress.map.delete(id);
            // 还有其他第二段爆炸 状态置为false
            if (progress.map.size) {
                progress.isExploded = false;
            }
            else {
                globalState.waitSecondPart = false;
            }
        })
    }

    /**
     * 处理炸弹引爆核心逻辑
     * @param id 炸弹实体ID
     * @param bombType 炸弹类型
     * @param cfg 炸弹配置数据
     * @param globalState 全局状态
     * @param timeout 定时器
     * @param animList 动画列表
     * @param bombList 炸弹列表
     * @param explode 爆炸计数器
     * @param frame 帧时间
     * @param lockTypeC 锁定类型组件
     * @param typeC 实体类型组件
     * @param stateC 状态组件
     * @param dataC 数据组件
     * @param sizeC 尺寸组件
     * @param progress 炸弹进度数据
     */
    dealBomb(
        id: number,
        bombType: BombType,
        cfg: BombTypeCfg,
        globalState: GlobalState,
        timeout: Timeout,
        animList: AnimList,
        bombList: BombList,
        explode: ExplodeCount,
        frame: FrameTime,
        lockTypeC,
        typeC,
        stateC,
        dataC,
        sizeC,
        progress: BombProgress
        , goalC
        , timeStep
    ) {
        let data = dataC.get(id);
        let [x, y] = data.pos;
        let [w, h] = globalState.areaMap.get(globalState.curArea);
        let indexs = globalState.areaIndexs.get(globalState.curArea);
        let time = cfg.wireTime;
        // 引燃导线
        animList.addAnimData(id, {
            type: AnimType.BombWire,
            time: cfg.wireTime,
            arg: bombType,
            posList: [[x, y]]
        });
        // 加上飞行时间
        if (animList.cur && animList.cur.type === AnimType.Flying) time += animList.cur.time;
        // 至少间隔一帧处理
        if (time < frame.frameInterval) time = frame.frameInterval
        timeout.value = new Timer(time, () => {
            let list = [];
            for (let [i, j] of cfg.area) {
                list.push([x + i, y + j]);
            }
            // 横雷
            if (cfg.effect.row) {
                this.rowExpansion(list, indexs, x, y, w, lockTypeC);
                if (cfg.effect.row === 3) { // 三横雷
                    if (y - 1 >= 0) this.rowExpansion(list, indexs, x, y - 1, w, lockTypeC);
                    if (y + 1 < h) this.rowExpansion(list, indexs, x, y + 1, w, lockTypeC);
                }
            }
            // 竖雷
            if (cfg.effect.column) {
                this.columnExpansion(list, indexs, x, y, h, lockTypeC)
                if (cfg.effect.column === 3) { // 三竖雷
                    if (x - 1 >= 0) this.columnExpansion(list, indexs, x - 1, y, h, lockTypeC);
                    if (x + 1 < w) this.columnExpansion(list, indexs, x + 1, y, h, lockTypeC);
                }
            }
            // 数量最多的物品
            if (cfg.effect.maxType) {
                let maxList = this.maxCount(indexs, w, h, typeC);
                list.push(...maxList);
            }
            let realPoscount = 0;
            for (let [x, y] of list) {
                if (x < 0 || x >= w || y < 0 || y >= h) continue;
                let id = indexs[LayerType.Lock][x][y];
                // 空地不算格子
                if (id && typeC.get(id).isEmpty()) continue;
                realPoscount++;
            }
            let otherTime = this.readyBomb(id, list, w, h, indexs, cfg, animList, timeout, bombList, globalState, stateC, typeC, dataC, goalC, timeStep, false, data.pos, undefined, undefined, sizeC);
            // 通知爆炸格子数  如果只有一个则是飞机或火箭，合并到第二部分通知
            if (realPoscount > 1) {
                let effects = [cfg.effect.priority, cfg.effect.random, cfg.effect.randomRow, cfg.effect.randomRow, cfg.effect.randomColumn, cfg.effect.randomNine];
                explode.isEnd = effects.every(v => !v) && bombList.waitBombs.length === 0;
                explode.value = realPoscount;
                realPoscount = 0;
            }
            // 有飞行时间 则有第二段爆炸
            if (cfg.flyTime > 0 || cfg.bombTime2 > 0) {
                let data: FirstBombData = {
                    lastList: list,
                    lastBomb: [x, y, id],
                    lastBombCfg: cfg,
                    realPoscount,
                    otherTime
                }
                progress.map.set(id, data);
                globalState.waitSecondPart = true;
            }
        })
    }

    /**
     * 横向扩展爆炸范围
     * @param list 当前爆炸位置列表
     * @param indexs 地图索引数据
     * @param x 起始X坐标
     * @param y 起始Y坐标
     * @param w 地图宽度
     * @param lockTypeC 锁定类型组件
     * @param tempList 临时存储列表（用于递归）
     * @returns 扩展后的位置列表
     */
    rowExpansion(
        list: number[][],
        indexs: number[][][],
        x: number,
        y: number,
        w: number,
        lockTypeC,
        tempList: number[][] = []
    ): number[][] {
        // 往右
        for (let i = x; i < w; i++) {
            if (list.find(v => v[0] === i && v[1] === y)) continue;
            list.push([i, y]);
            tempList.push([i, y])
            let id = indexs[LayerType.Lock][i][y];
            if (id && lockTypeC.get(id)?.blockBombExpansion) break;
        }
        // 往左
        for (let i = x - 1; i >= 0; i--) {
            if (list.find(v => v[0] === i && v[1] === y)) continue;
            list.push([i, y]);
            tempList.push([i, y])
            let id = indexs[LayerType.Lock][i][y];
            if (id && lockTypeC.get(id)?.blockBombExpansion) break;
        }
        return tempList;
    }

    /**
     * 竖向扩展爆炸范围
     * @param list 当前爆炸位置列表
     * @param indexs 地图索引数据
     * @param x 起始X坐标
     * @param y 起始Y坐标
     * @param h 地图高度
     * @param lockTypeC 锁定类型组件
     * @param tempList 临时存储列表（用于递归）
     * @returns 扩展后的位置列表
     */
    columnExpansion(
        list: number[][],
        indexs: number[][][],
        x: number,
        y: number,
        h: number,
        lockTypeC,
        tempList: number[][] = []
    ): number[][] {
        // 往上
        for (let j = y; j < h; j++) {
            if (list.find(v => v[0] === x && v[1] === j)) continue;
            list.push([x, j]);
            tempList.push([x, j])
            let id = indexs[LayerType.Lock][x][j];
            if (id && lockTypeC.get(id)?.blockBombExpansion) break;
        }
        // 往下
        for (let j = y - 1; j >= 0; j--) {
            if (list.find(v => v[0] === x && v[1] === j)) continue;
            list.push([x, j]);
            tempList.push([x, j])
            let id = indexs[LayerType.Lock][x][j];
            if (id && lockTypeC.get(id)?.blockBombExpansion) break;
        }
        return tempList;
    }

    /**
     * 统计地图上数量最多的物品类型
     * @param indexs 地图索引数据
     * @param w 地图宽度
     * @param h 地图高度
     * @param typeC 类型组件
     * @returns 最多物品的位置列表
     */
    maxCount(
        indexs: number[][][],
        w: number,
        h: number,
        typeC
    ): number[][] {
        let map = new Map, maxType = 0, maxList = [];
        for (let x = 0; x < w; x++) {
            for (let y = 0; y < h; y++) {
                let id = indexs[LayerType.Lock][x][y];
                if (id) continue;
                id = indexs[LayerType.Data][x][y];
                if (!id) continue;
                let type = typeC.get(id).value;
                let arr = map.get(type) || [];
                arr.push([x, y]);
                map.set(type, arr);
            }
        }
        for (let [type, arr] of map) {
            if (arr.length > maxList.length) {
                maxType = type;
                maxList = arr;
            }
        }

        return maxList;
    }

    /**
     * 准备爆炸动画和效果
     * @param entity 实体ID
     * @param list 爆炸位置列表
     * @param w 地图宽度
     * @param h 地图高度
     * @param indexs 地图索引
     * @param cfg 炸弹配置
     * @param animList 动画列表
     * @param timeout 定时器
     * @param bombList 炸弹列表
     * @param globalState 全局状态
     * @param stateC 状态组件
     * @param typeC 类型组件
     * @param dataC 数据组件
     * @param needFlying 是否需要飞行动画
     * @param startPos 起始位置（可选）
     * @param rocketEndPos 火箭终点位置（可选）
     * @param otherEndPos 其他终点位置（可选）
     * @param sizeC 尺寸组件
     * @returns 动画总时长
     */
    readyBomb(
        entity: number,
        list: number[][],
        w: number,
        h: number,
        indexs: number[][][],
        cfg: BombTypeCfg,
        animList: AnimList,
        timeout: Timeout,
        bombList: BombList,
        globalState: GlobalState,
        stateC,
        typeC,
        dataC,
        goalC,
        timeStep: TimeStep,
        needFlying: boolean = false,
        startPos?: number[],
        rocketEndPos?: number[][],
        otherEndPos?: number[][][],
        sizeC?
    ) {
        let posList = [], idList = [], baseIdList = [], cannotBombList = [];
        for (let [x, y] of list) {
            if (x < 0 || x >= w || y < 0 || y >= h) continue;
            let id = indexs[LayerType.Lock][x][y];
            if (!id) id = indexs[LayerType.Data][x][y];
            // 草垛可以被炸
            if (!id) id = indexs[LayerType.Lawn][x][y];
            if (id <= 0) continue;
            let type: TypeC = typeC.get(id);
            // 不能炸的类型
            if (type.cannotBomb) {
                cannotBombList.push(id);
                continue;
            }
            // 定时奖励刚创建出来的回合不允许被炸
            if (type.value === DataType.TimeAwardData && goalC.get(id).condition.start === timeStep.step) continue
            // // 等待爆炸的合成炸弹 不在此处引爆
            // if (bombList.waitBombs.includes(id)) continue;
            // 占地不止一格的物品只能记录一次
            let state = stateC.get(id);
            if ((idList.includes(id) || baseIdList.includes(id)) && (sizeC.get(id)?.width !== 1 || sizeC.get(id)?.height !== 1)) continue;
            // 等待爆炸的合成炸弹被引爆后，从等待爆炸中移出
            const index = bombList.waitBombs.findIndex(v => v === id);
            if (index >= 0) bombList.waitBombs.splice(index, 1);

            if (state.value !== DataState.WaitExplode && state.value !== DataState.Exploded) state.value = DataState.WaitExplode;
            posList.push([x, y, id])
            if (type.value > DataType.EmptyBox) {
                idList.push(id)
            } else {
                baseIdList.push(id)
            }
        }
        // 记录不能炸的物品
        globalState.cannotBombList.push(...cannotBombList);
        let time = 0, bombTime = cfg.bombTime;
        if (needFlying) {
            // 添加飞行动画
            animList.addAnimData(entity, {
                type: AnimType.Flying,
                time: cfg.flyTime,
                arg: cfg.id,
                posList: list,
                startPos: [startPos],
                rocketEndPos,
                otherEndPos
            })
            time += cfg.flyTime;
            bombTime = cfg.bombTime2;
        }
        animList.addAnimData(entity, {
            type: AnimType.Exploding,
            time: bombTime,
            arg: cfg.id,
            posList,
            startPos: [startPos],
            rocketEndPos,
            otherEndPos,
            entityKey: entity,
            isAnnex: false
        });
        let otherTime = 0;
        // 添加爆炸动画
        if (baseIdList.length > 0) {
            animList.addAnimData(entity, {
                type: AnimType.ExplodingAni,
                time: AnimTime.ExplodingAni,
                arg: cfg.id,
                posList,
                startPos: [startPos],
                rocketEndPos,
                otherEndPos,
                entityKey: entity,
                isAnnex: false
            });
            otherTime = AnimTime.ExplodingAni;
        }
        time += bombTime;
        // 处理其他物件
        timeout.value = new Timer(time, () => {
            // 清空飞机选择的目标
            if (needFlying) this.airplanePosList = [];
        })

        return otherTime;
    }

    // 随机选目标
    random(list: number[][], count: number, indexs: number[][][], rand: Rand, w: number, h: number, typeC) {
        let posList = [];
        for (let x = 0; x < w; x++) {
            for (let y = 0; y < h; y++) {
                // 排除已选区域
                if (list.find(v => v[0] === x && v[1] === y)) continue;
                let id = indexs[LayerType.Lock][x][y];
                // 排除不能被炸的物品
                if (id && typeC.get(id).value === DataType.SpecialLock) continue;
                // 排除飞机选择的格子
                if (this.airplanePosList.find(v => v[0] === x && v[1] === y)) continue;
                posList.push([x, y]);
            }
        }
        let secondList = []
        if (posList.length <= 0) return secondList;
        for (let i = 0; i < count; i++) {
            let pos = randomPos(posList, rand);
            secondList.push(pos);
        }
        return secondList;
    }

    // 优先级随机
    priorityRandom(list: number[][], count: number, indexs: number[][][], rand: Rand, typeC, dataC, collect: Collect, bombList: BombList) {
        let priority = [[], [], [], [], [], [], [], []];
        for (let type of typeC.iter()) {
            if (!type) continue;
            let id = (<any>type).__id__;
            let [x, y] = dataC.get(id).pos;
            // 排除当前已炸区域
            if (list.find(v => v[0] === x && v[1] === y) || this.airplanePosList.find(v => v[0] === x && v[1] === y)) continue;
            // // 排除该位置有等待爆炸的物件
            // let dataId = indexs[LayerType.Data][x][y];
            // if (dataId && bombList.waitBombs.includes(dataId)) continue

            // 如果优先炸事件有配置，则放入最后
            const perBoomType = collect.perBoomType.find(v => v[0] === type.subType || v[0] === type.value)
            if (perBoomType && !type.cannotBomb) {
                priority[5].push([x, y, perBoomType[1]]);
            }
            // 排除有锁的格子
            if (indexs[LayerType.Lock][x][y]) continue;
            if (type.value === DataType.EventData) {
                let cfg = getCfg(EventDataCfg, type.subType);
                if (cfg?.priority) {
                    priority[cfg.priority - 1].push([x, y]);
                }
            }
            // 普通袋子 > 英雄
            else if (type.hideType === AwardType.NormalBag) {
                priority[3].push([x, y]);
            }// 定时宝箱
            else if (type.value === DataType.TimeAwardData) {
                priority[6].push([x, y]);
            }
            else if (!type.cannotBomb) {
                priority[7].push([x, y]);
            }
        }
        // console.log('排除的坐标', JSON.stringify(this.airplanePosList))
        let posList = [];
        while (posList.length < count) {
            let temp = 0;
            for (let arr of priority) {
                if (arr.length === 0) continue;
                temp++;
                let random;
                if (arr[0][2]) {// 走权重
                    const weights = arr.map(v => v[2]);
                    random = this.randByWeight(weights, rand.nextFloat(0, 1));
                } else {
                    random = rand.nextInt(0, arr.length - 1);
                }

                this.airplanePosList.push(arr[random]);
                posList.push(arr[random]);
                arr.splice(random, 1);
                break;
            }

            if (temp <= 0) break
        }
        console.log('优先级随机', posList);

        return posList
    }

    /**
     * @description 根据权重产生随机下标
     * @param seedRate 种子比例(0~1)
     */
    private randByWeight(weights: number[], seedRate: number): number {
        let temp = 0;
        const sum = weights.reduce((p, c) => p + c, 0)
        const rand = sum * seedRate;
        for (let i = 0; i < weights.length; i++) {
            if (rand >= temp && rand <= (temp + weights[i])) {
                return i;
            }
            temp += weights[i];
        }
    }

    // 随机几行
    randomRow(indexs: number[][][], count: number, w: number, h: number, rand: Rand, pos: number[], typeC) {
        let yList = [];
        for (let y = 0; y < h; y++) {
            // 排除自身所在行
            if (y === pos[1]) continue;
            yList.push(y);
        }
        let posList = [];
        while (posList.length < count) {
            let ind = rand.nextInt(0, yList.length - 1);
            let y = yList[ind];
            yList.splice(ind, 1)
            let xList = [];
            for (let x = 0; x < w; x++) {
                let id = indexs[LayerType.Lock][x][y];
                // 排除不能被炸的物品
                if (id && typeC.get(id).value === DataType.SpecialLock) continue;
                xList.push(x);
            }
            if (xList.length > 0) {
                let ind1 = rand.nextInt(0, xList.length - 1);
                let x = xList[ind1];
                posList.push([x, y])
            }
        }

        return posList;
    }

    // 随机几列
    randomColumn(indexs: number[][][], count: number, w: number, h: number, rand: Rand, pos: number[], typeC) {
        let xList = []
        for (let x = 0; x < w; x++) {
            // 排除自身所在列
            if (x === pos[0]) continue;
            xList.push(x);
        }
        let posList = [];
        while (posList.length < count) {
            let ind = rand.nextInt(0, xList.length - 1);
            let x = xList[ind];
            xList.splice(ind, 1);
            let yList = [];
            for (let y = 0; y < h; y++) {
                let id = indexs[LayerType.Lock][x][y];
                // 排除不能被炸的物品
                if (id && typeC.get(id).value === DataType.SpecialLock) continue;
                yList.push(y);
            }
            if (yList.length > 0) {
                let ind1 = rand.nextInt(0, yList.length - 1);
                let y = yList[ind1];
                posList.push([x, y])
            }
        }

        return posList;
    }

    // 第二部分 飞机和火箭和全屏雷
    secondPart(list: number[][], indexs: number[][][], cfg: BombTypeCfg, w: number, h: number, pos: number[], rand: Rand, typeC, dataC, lockTypeC
        , collect: Collect, bombList: BombList) {
        let secondList = [];
        let rocketEndPos = []; // 火箭终点目标
        let otherEndPos = []; // 其他终点目标
        let realPoscount = 0; // 爆炸格子数
        // 优先级选坐标  两次怪＞普通怪＞金袋＞剑盾＝藏有炸弹袋的格子
        if (cfg.effect.priority) {
            let posList = this.priorityRandom(list, cfg.effect.priority, indexs, rand, typeC, dataC, collect, bombList);
            for (let [x, y] of posList) {
                let tempList = [];
                for (let [i, j] of cfg.effect.area) {
                    let x1 = x + i, y1 = y + j;
                    if (x < 0 || x >= w || y < 0 || y >= h) continue;
                    let id = indexs[LayerType.Lock][x][y];
                    // 空地不算格子
                    if (id && typeC.get(id).isEmpty()) continue;
                    realPoscount++;
                    // if (secondList.find(v => v[0] === x1 && v[1] === y1)) continue;
                    secondList.push([x1, y1]);
                    tempList.push([x1, y1])
                }
                // 触发横雷
                if (cfg.effect.afterRow) {
                    let tmp = this.rowExpansion(secondList, indexs, x, y, w, lockTypeC)
                    realPoscount += tmp.length;
                    tempList.push(...tmp);
                }
                // 触发竖雷
                if (cfg.effect.afterColumn) {
                    let tmp = this.columnExpansion(secondList, indexs, x, y, h, lockTypeC)
                    realPoscount += tmp.length;
                    tempList.push(...tmp);
                }
                // 火箭
                if (cfg.id === BombType.SplitBomb) {
                    rocketEndPos.push(...tempList);
                } else {
                    otherEndPos.push(tempList);
                    this.airplanePosList.push(...tempList);
                }
            }
        }
        // 随机选目标
        if (cfg.effect.random) {
            let posList = this.random(list, cfg.effect.random, indexs, rand, w, h, typeC);
            for (let [x, y] of posList) {
                if (secondList.find(v => v[0] === x && v[1] === y)) continue;
                secondList.push([x, y])
            }
            realPoscount += posList.length;
            rocketEndPos.push(...posList);

        }
        // 随机几行
        if (cfg.effect.randomRow) {
            let posList = this.randomRow(indexs, cfg.effect.randomRow, w, h, rand, pos, typeC);
            for (let [x, y] of posList) {
                let tempList = this.rowExpansion(secondList, indexs, x, y, w, lockTypeC)
                otherEndPos.push(tempList);
                realPoscount += tempList.length;
            }

        }
        // 随机几列
        if (cfg.effect.randomColumn) {
            let posList = this.randomColumn(indexs, cfg.effect.randomColumn, w, h, rand, pos, typeC);
            for (let [x, y] of posList) {
                let tempList = this.columnExpansion(secondList, indexs, x, y, h, lockTypeC);
                otherEndPos.push(tempList);
                realPoscount += tempList.length;
            }
        }
        // 随机九宫
        if (cfg.effect.randomNine) {
            const outList = [];
            for (let x = 0; x < w; x++) {
                if (x <= 0 || x >= (w - 1)) {
                    for (let y = 0; y < h; y++) {
                        outList.push([x, y]);
                    }
                } else {
                    outList.push([x, 0]);
                    if (h > 1) outList.push([x, h - 1]);
                }
            }
            // 优先抛掉外圈，如果选不出来，则全选择
            let posList = this.random(list.concat(outList), cfg.effect.randomNine, indexs, rand, w, h, typeC);
            if (posList.length <= 0) {
                posList = this.random(list, cfg.effect.randomNine, indexs, rand, w, h, typeC);
            }
            for (let [x, y] of posList) {
                let tempList = [];
                secondList.push([x, y]);
                tempList.push([x, y])
                for (let [i, j] of AroundPosEight) {
                    let x1 = x + i, y1 = y + j;
                    if (x < 0 || x >= w || y < 0 || y >= h) continue;
                    let id = indexs[LayerType.Lock][x][y];
                    // 空地不算格子
                    if (id && typeC.get(id).isEmpty()) continue;
                    realPoscount++;
                    // if (secondList.find(v => v[0] === x1 && v[1] === y1)) continue;
                    secondList.push([x1, y1])
                    tempList.push([x1, y1])
                }
                otherEndPos.push(tempList);
            }
        }
        //全屏
        if (cfg.effect.full_screen) {
            secondList.push([pos[0], pos[1]]);
            for (let i = 0; i < w; i++) {
                for (let ii = 0; ii < h; ii++) {
                    if (i === pos[0] && ii === pos[1]) continue;
                    let id = indexs[LayerType.Lock][i][ii];
                    // 空地不算格子
                    if (id && typeC.get(id).isEmpty()) continue;
                    realPoscount++;
                    secondList.push([i, ii]);
                }
            }
            otherEndPos.push(secondList);
        }
        console.log('第二阶段爆炸坐标', rocketEndPos, otherEndPos, realPoscount);

        return { secondList, rocketEndPos, otherEndPos, realPoscount };
    }

    // 取出新创建的定时奖励
    private getPosFromNewTimeAwardData(timeStep: TimeStep, goalC: ArrMulti<Entity, GoalC>, dataC: ArrMulti<Entity, GoalC>) {
        const list: number[][] = []
        timeStep.list.forEach(id => {
            if (goalC.get(id).condition.start !== timeStep.step) return
            let [x, y] = dataC.get(id).pos;
            list.push([x, y]);
        })

        return list
    }

}