import { Entity, ArrMulti, Single, System, World, read, listenModify, ModifyEvent, Multi, listenCreate } from "pi_ucore/util/ecs";
import { DataC, DataState, LayerType, StateC } from "../component/data";
import { GameState, GlobalState } from "../single/global_state";
import { Collect } from "../single/collect";
import { destroyOneData } from "../utils/init_match";
import { GoalC, GoalType } from "../component/goal";
import { AnimList, AnimTime, AnimType } from "../single/anim_list";
import { Timeout, Timer } from "pi_scene/scene/single/global";
import { DataType, TypeC } from "../component/type";
import { BombType } from "../single/bomb_list";
/**
 * 收集系统 - 处理游戏物品收集逻辑和目标追踪
 * 
 * 主要职责：
 * 1. 追踪各种游戏物品的收集（樱桃、甜甜圈、炸弹等）
 * 2. 处理特殊收集条件（锁定层、合成目标）
 * 3. 管理收集相关动画和时序
 * 4. 更新全局收集计数器和目标进度
 * 5. 与动画系统协调视觉反馈
 */
export class CollectSys implements System {

    /**
     * 系统初始化方法（ECS生命周期可选方法）
     * @param _world - ECS世界实例
     */
    setup(_world: World) {
    }

    /**
     * 处理可收集实体的状态变化
     * @param e - 包含实体ID的修改事件
     * @param read - 读取组件元组：
     *   [0] GlobalState - 当前游戏状态
     *   [1] AnimList - 活动动画注册表
     *   [2] DataC[] - 位置数据组件
     *   [3] StateC[] - 状态组件
     *   [4] TypeC[] - 类型组件
     *   [5] Collect - 全局收集追踪器
     *   [6] Timeout - 计时器系统
     * @param write - ECS写操作
     */
    @listenModify([Entity, StateC])
    @read(GlobalState, AnimList, [Entity, DataC], [Entity, StateC], [Entity, TypeC], Collect, Timeout)
    listenState(e: ModifyEvent, read: [Single<GlobalState>, Single<AnimList>, ArrMulti<Entity, DataC>, ArrMulti<Entity, StateC>, ArrMulti<Entity, TypeC>,
        Single<Collect>, Single<Timeout>], write: any) {

        let globalState = read[0].data;
        let animList = read[1].data;
        let dataC = read[2];
        let stateC = read[3];
        let typeC = read[4];
        let collect = read[5].data;
        let timeout = read[6].data;
        let data = dataC.get(e.id);
        let state = stateC.get(e.id);
        let type = typeC.get(e.id);
        let [x, y] = data.pos;
        let indexs = globalState.areaIndexs.get(globalState.curArea);

        // 处理锁定层销毁
        if (state.value === DataState.Exploded && data.layer === LayerType.Lock && !type.needBombTimes) {
            let id = indexs[LayerType.Data][x][y];
            // 如果物品层是樱桃 直接收集
            if (id && typeC.get(id).value === DataType.Cherry) {
                collect.addCount(DataType.Cherry, 1);
                destroyOneData(write, id);
                indexs[LayerType.Data][x][y] = 0;
            }
        }
        // 甜甜圈掉落到最底部 或 与水域相连
        if (state.value === DataState.Normal && data.layer === LayerType.Data && type.value === DataType.Donut && (y === 0 || indexs[LayerType.Lawn][x][y] < 0)) {
            // 甜甜圈动画(鱼跳水)
            animList.addAnimData(e.id, {
                type: AnimType.FishJump,
                time: AnimTime.FishJump,
                posList: [data.pos]
            });
            state.value = DataState.WaitExplode;
            timeout.value = new Timer(AnimTime.FishJump, (id) => {
                type.needBombTimes--;
                state.value = DataState.Exploded;
                console.log('掉到底部, 销毁', id, data.pos);
            }, e.id)
            return;
        }

        // 处理炸弹收集
        if (state.value === DataState.WaitExplode && type.value === DataType.SynthesisBomb) {
            if (type.subType > BombType.ChainBomb) {
                collect.addCount(type.value, 1); // Count main bomb type
                collect.addCount(type.subType, 1); // Count sub-type
            }
            return;
        }

        // 处理常规物品收集
        if (state.value === DataState.Exploded && !type.needBombTimes) {
            if (type.value === DataType.TimeAwardData) {// 定时奖励只有通过正常爆炸引爆才会记录收集
                if (state.oldValue === DataState.WaitExplode) {
                    collect.addCount(type.value, 1); // Add main type
                    if (type.subType) collect.addCount(type.subType, 1); // Add subtype if exists
                }
            } else {
                collect.addCount(type.value, 1); // Add main type
                if (type.subType) collect.addCount(type.subType, 1); // Add subtype if exists
            }

        }
    }

    /**
     * 处理新可收集类型的创建
     * @param e - 实体创建的修改事件
     * @param read - 读取组件元组：
     *   [0] TypeC[] - 类型组件
     *   [1] AnimList - 动画注册表
     *   [2] Collect - 全局收集追踪器
     *   [3] Timeout - 计时器系统
     * @param write - ECS写操作
     */
    @listenCreate([Entity, TypeC])
    @read([Entity, TypeC], AnimList, Collect, Timeout)
    listenCreateType(e: ModifyEvent, read: [ArrMulti<Entity, TypeC>, Single<AnimList>, Single<Collect>, Single<Timeout>], write: any) {
        let typeC = read[0];
        let type = typeC.get(e.id);
        let collect = read[2].data;
        if (type.value === DataType.SynthesisBomb) {
            collect.addCount(type.value, 1); // Count main type
            if (type.subType) collect.addCount(type.subType, 1); // Count sub-type
            return;
        }
    }

    /**
     * 处理目标相关的收集逻辑
     * @param e - 目标变化的修改事件
     * @param read - 读取组件元组：
     *   [0] Collect - 全局收集追踪器
     *   [1] GoalC[] - 目标组件
     *   [2] StateC[] - 状态组件
     *   [3] AnimList - 动画注册表
     *   [4] DataC[] - 位置数据
     *   [5] Timeout - 计时器系统
     * @param write - ECS写操作
     */
    @listenModify([Entity, GoalC])
    @read(Collect, [Entity, GoalC], [Entity, StateC], AnimList, [Entity, DataC], Timeout)
    listenGoal(e: ModifyEvent, read: [Single<Collect>, Multi<Entity, GoalC>, ArrMulti<Entity, StateC>, Single<AnimList>, ArrMulti<Entity, DataC>, Single<Timeout>], write: any) {
        let collect = read[0].data;
        let goalC = read[1];
        let stateC = read[2];
        let animList = read[3].data;
        let dataC = read[4];
        let timeout = read[5].data;
        let goal = goalC.get(e.id);

        // 处理产出目标（物品生产）
        if (goal.type === GoalType.OutPut) {
            let num = 0;
            if (goal.collectStep) {
                // 基于步骤的收集（例如每收集10个物品奖励一次）
                let old = collect.dataList.get(goal.dataType) || 0;
                let newNum = Math.floor(Math.min(goal.dataCount, goal.goalCount) / goal.collectStep);
                num = newNum - old;
            } else {
                // 简单增量收集
                num = Math.min(goal.dataCount, goal.goalCount) - Math.min(goal.oldDataCount, goal.goalCount);
            }
            if (num > 0) {
                collect.addCount(goal.dataType, num);
            }
        }
        // 处理合成目标（物品组合）
        else if (goal.type === GoalType.Synthesis) {
            let posList = [];
            // 触发合成动画
            animList.addAnimData(e.id, {
                type: AnimType.SynthesisData,
                time: AnimTime.SynthesisData,
                posList: posList,
                dataType: goal.dataType
            });
            timeout.value = new Timer(AnimTime.SynthesisData, () => {
                // 动画结束后销毁组件
                for (let id of goal.idList) {
                    let [x, y] = dataC.get(id).pos;
                    stateC.get(id).value = DataState.Exploded;
                    posList.push([x, y]);
                }
                collect.addCount(goal.dataType, 1);
            });
        }
    }

    /**
     * 处理影响收集的全局状态变化
     * @param e - 全局状态修改事件
     * @param read - 读取组件元组：
     *   [0] GlobalState - 游戏状态
     *   [1] Collect - 收集追踪器
     *   [2] AnimList - 动画注册表
     *   [3] GoalC[] - 目标组件
     *   [4] StateC[] - 状态组件
     *   [5] DataC[] - 位置数据
     *   [6] Timeout - 计时器系统
     * @param write - ECS写操作
     */

    @listenModify(GlobalState)
    @read(GlobalState, Collect, AnimList, [Entity, GoalC], [Entity, StateC], [Entity, DataC], Timeout)
    listenGlobalState(e: ModifyEvent, read: [Single<GlobalState>, Single<Collect>, Single<AnimList>, Multi<Entity, GoalC>, ArrMulti<Entity, StateC>, ArrMulti<Entity, DataC>, Single<Timeout>],
        write: any) {

        let globalState = read[0].data;
        let collect = read[1].data;
        let animList = read[2].data;
        let goalC = read[3];
        let stateC = read[4];
        let dataC = read[5];
        let timeout = read[6].data;
        // 处理动画结束状态
        if (globalState.gameState === GameState.AnimEnd) {
            for (let [id, goal] of goalC.iter()) {
                if (goal.type === GoalType.Collect) {
                    // 物品收集成功时销毁自身
                    if (collect.dataList.get(goal.dataType) >= goal.goalCount && stateC.get(id).value === DataState.Normal) {
                        let data = dataC.get(id);
                        // 触发关窗帘动画
                        animList.addAnimData(id, {
                            type: AnimType.CloseCurtains,
                            time: AnimTime.CloseCurtains,
                            posList: [data.pos]
                        })
                        stateC.get(id).value = DataState.WaitExplode;
                        timeout.value = new Timer(AnimTime.CloseCurtains, (id) => {
                            stateC.get(id).value = DataState.Exploded;
                            console.log('窗帘关闭完成，销毁实体', id, data.pos);
                        }, id)
                    }
                }
            }
        }
    }
}
