import { ArrMulti, Entity, ModifyEvent, Multi, Single, System, World, listenModify, read } from "pi_ucore/util/ecs";
import { GameState, GlobalState } from "../single/global_state";
import { DataC, DataState, LayerType, StateC } from "../component/data";
import { BombList } from "../single/bomb_list";
import { AroundPosFour, randomPos } from "../utils/base";
import { Calc } from "pi_common_tools/util/calc";
import { createOneData } from "../utils/init_match";
import { Timeout, Timer } from "pi_scene/scene/single/global";
import { DataType, TypeC } from "../component/type";
import { GoalC, GoalType } from "../component/goal";
import { Rand } from "pi_math/rand";
import { AnimList, AnimTime, AnimType } from "../single/anim_list";

/**
 * 扩展系统 - 处理游戏中的地块扩展逻辑
 * 
 * 主要职责：
 * 1. 监听炸弹使用事件，处理炸弹爆炸后的地块扩展
 * 2. 实现两种扩展模式：
 *    - 周围扩展：炸弹爆炸后向四周可扩展区域延伸
 *    - 随机扩展：根据条件随机选择区域进行扩展
 * 3. 管理美杜莎石化动画效果
 * 4. 处理游戏状态转换与扩展时机的协调
 */
export class ExpandSys implements System {
    /** 标记是否使用了炸弹 */
    useBomb: boolean = false;
    
    /** 标记是否阻止扩展（当爆炸影响到可扩展目标时） */
    stopExpand: boolean = false;

    /**
     * 系统初始化方法（可选）
     * @param _world - 实体世界上下文
     */
    setup(_world: World) {
    }

    /**
     * 监听炸弹列表修改事件
     * @param e - 修改事件对象
     * @param read - 读取的炸弹列表数据
     */
    @listenModify(BombList)
    @read(BombList)
    listenBombList(e: ModifyEvent, read: [Single<BombList>]) {
        if (e.field !== 'times') return; // 只关注times字段变化
        
        let bombList = read[0].data;
        if (bombList.times) {
            this.useBomb = true; // 标记已使用炸弹
        }
    }

    /**
     * 监听实体状态变化事件
     * @param e - 修改事件对象
     * @param read - 读取的组件数据集合，包含：
     *   [0] GlobalState 全局状态
     *   [1] DataC 数据组件
     *   [2] StateC 状态组件
     *   [3] TypeC 类型组件
     *   [4] GoalC 目标组件
     */
    @listenModify([Entity, StateC])
    @read(GlobalState, [Entity, DataC], [Entity, StateC], [Entity, TypeC], [Entity, GoalC])
    listenState(e: ModifyEvent, read: [Single<GlobalState>, ArrMulti<Entity, DataC>, ArrMulti<Entity, StateC>, ArrMulti<Entity, TypeC>,
        Multi<Entity, GoalC>], write: any) {
        let globalState = read[0].data;
        let dataC = read[1];
        let stateC = read[2];
        let typeC = read[3];
        let goalC = read[4];
        
        // 获取当前实体的状态、类型和目标信息
        let state = stateC.get(e.id);
        let type = typeC.get(e.id);
        let goal = goalC.get(e.id);
        
        // 当实体爆炸且满足以下条件时阻止扩展：
        // 1. 状态为已爆炸
        // 2. 已没有炸弹次数
        // 3. 目标类型为周围扩展
        if (state.value === DataState.Exploded 
            && !type.needBombTimes 
            && goal?.type === GoalType.AroundExpand) {
            this.stopExpand = true;
        }
    }

    /**
     * 监听全局状态变化事件（主逻辑入口）
     * @param e - 修改事件对象
     * @param read - 读取的组件数据集合，包含：
     *   [0] GlobalState 全局状态
     *   [1] Calc 计算工具
     *   [2] BombList 炸弹列表
     *   [3] GoalC 目标组件
     *   [4] TypeC 类型组件
     *   [5] DataC 数据组件
     *   [6] Timeout 超时控制
     *   [7] AnimList 动画列表
     */
    @listenModify(GlobalState)
    @read(GlobalState, Calc, BombList, [Entity, GoalC], [Entity, TypeC], [Entity, DataC], Timeout, AnimList)
    listenGlobalState(e: ModifyEvent, read: [Single<GlobalState>, Single<Calc>, Single<BombList>, Multi<Entity, GoalC>, ArrMulti<Entity, TypeC>, ArrMulti<Entity, DataC>, Single<Timeout>, Single<AnimList>],
        write: any) {
        let globalState = read[0].data;
        let calc = read[1].data;
        let bombList = read[2].data;
        let goalC = read[3];
        let typeC = read[4];
        let dataC = read[5];
        let timeOut = read[6].data;
        let animList = read[7].data;
        
        // 获取当前区域索引和地图尺寸
        let indexs = globalState.areaIndexs.get(globalState.curArea);
        let [w, h] = globalState.areaMap.get(globalState.curArea);

        // 在掉落结束状态且没有等待中的炸弹时处理扩展逻辑
        if (globalState.gameState === GameState.DropEnd && !bombList.waitBombs.length) {
            
            // 处理周围扩展逻辑
            for (let [id, goal] of goalC.iter()) {
                if (goal.type === GoalType.AroundExpand 
                    && this.useBomb 
                    && !this.stopExpand) {
                    let [x, y] = dataC.get(id).pos;
                    let fg = this.aroundExpand(write, indexs, globalState.curArea, goal, x, y, w, h)
                    if (fg) break; // 成功扩展一个方向后跳出
                }
            }

            // 准备随机扩展参数
            let index = 0;    // 当前操作次数
            let choosed = []; // 已选坐标缓存
            
            // 处理随机扩展逻辑
            for (let [id, goal] of goalC.iter()) {
                if (goal.type === GoalType.RandomExpand 
                    && this.useBomb 
                    && bombList.times % goal.condition.times === 0) {
                    let medusaKey = this.getMedusaId(globalState, typeC, index);
                    index++;
                    this.randomExpand(write, indexs, w, h, globalState.curArea, calc.rand, 
                        goal, typeC, animList, timeOut, medusaKey, globalState, choosed)
                }
            }

            // 重置状态标记
            this.stopExpand = false;
            this.useBomb = false;
        }
    }

    /**
     * 执行周围扩展逻辑
     * @param world - 实体世界上下文
     * @param indexs - 当前区域的三维索引数组 [层][x][y]
     * @param area - 当前区域ID
     * @param goal - 扩展目标配置
     * @param x - 源点x坐标
     * @param y - 源点y坐标
     * @param w - 区域宽度
     * @param h - 区域高度
     * @returns 是否成功扩展
     */
    aroundExpand(world: World, indexs: number[][][], area: number, goal: GoalC, 
               x: number, y: number, w: number, h: number): boolean {
        // 遍历四个方向（上、下、左、右）
        for (let [i, j] of AroundPosFour) {
            let x1 = x + i, y1 = y + j;
            
            // 边界检查
            if (x1 < 0 || x1 >= w || y1 < 0 || y1 >= h) continue;
            
            // 检查目标位置是否已被锁定
            let id = indexs[LayerType.Lock][x1][y1];
            if (id) continue; // 跳过已锁定位置
            
            // 创建新的锁定数据
            let newId = createOneData(world, [x1, y1], area, LayerType.Lock, 
                                    [1, 1], 1, goal.dataType, goal.condition.subType);
            indexs[LayerType.Lock][x1][y1] = newId;
            console.log('周围扩展成功 位置:', [x1, y1], '实体ID:', newId);
            return true; // 成功扩展一个方向后返回
        }
        return false;
    }




    /**
     * 执行随机扩展逻辑
     * @param world - 实体世界上下文
     * @param indexs - 当前区域的三维索引数组
     * @param w - 区域宽度
     * @param h - 区域高度 
     * @param area - 当前区域ID
     * @param rand - 随机数生成器
     * @param goal - 扩展目标配置
     * @param typeC - 类型组件集合
     * @param animList - 动画列表
     * @param timeOut - 超时控制器
     * @param medusaId - 美杜莎实体ID
     * @param globalState - 全局状态
     * @param choosed - 已选坐标缓存
     */
    randomExpand(world: World, indexs: number[][][], w: number, h: number, area: number, rand: Rand, goal: GoalC, typeC, animList: AnimList
        , timeOut: Timeout, medusaId: number, globalState: GlobalState, choosed: number[][]) {
        console.log('随机扩展目标', goal.goalCount);
        let posList = []
        for (let x = 0; x < w; x++) {
            for (let y = 0; y < h; y++) {
                // 不选被锁坐标
                let id = indexs[LayerType.Lock][x][y];
                if (id) continue;
                id = indexs[LayerType.Data][x][y];
                let list = [DataType.Medusa, DataType.EventData, DataType.AwardData, DataType.SynthesisBomb, DataType.TimeAwardData]
                // 不选空坐标 和 不可石化的目标
                if (!id || list.includes(typeC.get(id).value)) continue;
                // 已经选择的不再被选择
                if (choosed.some(v => v[0] === x && v[1] === y)) continue;
                posList.push([x, y]);
            }
        }
        let allPos: [number, number][] = [];
        if (posList.length > 0) {
            for (let i = 0; i < goal.goalCount; i++) {
                let pos = randomPos(posList, rand);
                allPos.push([pos[0], pos[1]])
                choosed.push([pos[0], pos[1]])
                console.log('随机扩展目标', pos);
            }
        }

        // 添加石化动画
        animList.addAnimData(medusaId, {
            type: AnimType.MedusaAttack,
            time: AnimTime.MedusaAttack,
            posList: allPos,
            entityKey: medusaId
        });
        timeOut.value = new Timer(AnimTime.MedusaAttack, () => {
            globalState.gameState = GameState.DropEnd;
            for (let [x, y] of allPos) {
                let id = createOneData(world, [x, y], area, LayerType.Lock, [1, 1], 1, goal.dataType, goal.condition.subType)
                indexs[LayerType.Lock][x][y] = id;
            }
        });
    }

    /**
     * 获取美杜莎实体ID
     * @param globalState - 全局状态
     * @param typeC - 类型组件集合
     * @param index - 当前索引（用于防止重复选择同一美杜莎）
     * @returns 美杜莎实体ID
     */
    getMedusaId(globalState: GlobalState, typeC: ArrMulti<Entity, TypeC>, index: number) {
        const areaIndexs: number[][][] = globalState.areaIndexs.get(globalState.curArea);
        let outKey = -1;
        for (let x = 0; x < areaIndexs[1].length; x++) {
            for (let y = 0; y < areaIndexs[1][x].length; y++) {
                let key = areaIndexs[1][x][y];
                if (!key) continue;
                let type = typeC.get(key);
                if (type.value !== DataType.Medusa) continue
                if (index <= 0) return key;
                outKey = key;
                index--;
            }
        }

        if (outKey >= 0) return outKey
    }
}
