import { Entity, ArrMulti, Single, System, World, read, listenModify, ModifyEvent, Multi, listenCreate } from "pi_ucore/util/ecs";
import { DataC, DataState, DirectionType, LayerType, StateC } from "../component/data";
import { GameState, GlobalState } from "../single/global_state";
import { MoveC } from "pi_scene/scene/component/common";
import { createOneData } from "../utils/init_match";
import { GoalC, GoalType } from "../component/goal";
import { Timeout, Timer } from "pi_scene/scene/single/global";
import { Calc } from "pi_common_tools/util/calc";
import { AnimList } from "../single/anim_list";
import { BombList, BombType } from "../single/bomb_list";
import { TimeStep } from "../single/time_step";
import { DataType, NormalLockType, TypeC } from "../component/type";

/**
 * 定时步骤系统
 */
export class TimeStepSys implements System {
    /** 标记是否需要处理掉落逻辑 */
    dirty: boolean = false;

    /** 系统初始化方法（ECS生命周期方法） */
    setup(_world: World) {
    }

    /**
     * 主运行逻辑 - 处理每帧的掉落检测和移动控制
     * @param read 只读数据集合 [
     *   GlobalState,   // 全局游戏状态
     *   Calc,          // 计算工具
     *   StateC[],      // 实体状态组件
     *   MoveC[],       // 移动控制组件
     *   DataC[],       // 实体数据组件
     *   GoalC[],       // 目标组件
     *   Timeout,       // 定时器
     *   AnimList       // 动画列表
     * ]
     */
    @read(GlobalState, Calc, [Entity, StateC], [Entity, MoveC], [Entity, DataC], [Entity, GoalC], Timeout, AnimList)
    run(read: [Single<GlobalState>, Single<Calc>, ArrMulti<Entity, StateC>, Multi<Entity, MoveC>, ArrMulti<Entity, DataC>, ArrMulti<Entity, GoalC>, Single<Timeout>
        , Single<AnimList>], write: any) {
    }

    /**
         * 处理可收集实体的状态变化
         * @param e - 包含实体ID的修改事件
         * @param read - 读取组件元组：
         *   [0] GlobalState - 当前游戏状态
         *   [1] AnimList - 活动动画注册表
         *   [2] DataC[] - 位置数据组件
         *   [3] StateC[] - 状态组件
         *   [4] TypeC[] - 类型组件
         *   [5] Collect - 全局收集追踪器
         *   [6] Timeout - 计时器系统
         * @param write - ECS写操作
         */
    @listenModify([Entity, StateC])
    @read([Entity, StateC], [Entity, TypeC], TimeStep)
    listenState(e: ModifyEvent, read: [ArrMulti<Entity, StateC>, ArrMulti<Entity, TypeC>, Single<TimeStep>], write: any) {
        let state: StateC = read[0].get(e.id);
        let type: TypeC = read[1].get(e.id);
        let timeStep: TimeStep = read[2].data;

        // 处理锁定层销毁
        if (state.value !== DataState.Exploded || type.value !== DataType.TimeAwardData) return
        timeStep.remove(e.id);
        // 消除的该物品需要统计收集
    }

    /**
     * 监听全局状态变化事件（主逻辑入口）
     * @param e - 修改事件对象
     * @param read - 读取的组件数据集合，包含：
     *   [0] GlobalState 全局状态
     *   [1] Calc 计算工具
     *   [2] BombList 炸弹列表
     *   [3] GoalC 目标组件
     *   [4] TypeC 类型组件
     *   [5] DataC 数据组件
     *   [6] Timeout 超时控制
     *   [7] AnimList 动画列表
     */
    @listenModify(GlobalState)
    @read(GlobalState, BombList, TimeStep, [Entity, GoalC], [Entity, StateC], [Entity, TypeC])
    listenGlobalState(e: ModifyEvent, read: [Single<GlobalState>, Single<BombList>, Single<TimeStep>, ArrMulti<Entity, GoalC>, ArrMulti<Entity, StateC>, ArrMulti<Entity, TypeC>],
        write: any) {
        let globalState: GlobalState = read[0].data;
        let bombList = read[1].data;
        let timeStep: TimeStep = read[2].data;
        let goalC = read[3];
        let stateC = read[4];
        let typeC = read[5];

        // 在掉落结束状态且没有等待中的炸弹时处理扩展逻辑
        if (globalState.gameState === GameState.DropEnd && !bombList.waitBombs.length) {
            // 检查自动爆炸节点
            this.checkAutoBoom(timeStep, goalC, stateC, typeC);
            // 记录新的步骤
            timeStep.step += 1;
        }
    }


    // 检查定时爆炸节点
    private checkAutoBoom(timeStep: TimeStep, goalC: ArrMulti<Entity, GoalC>, stateC: ArrMulti<Entity, StateC>, typeC: ArrMulti<Entity, TypeC>) {
        const newList = timeStep.list.filter(id => {
            const goal = goalC.get(id);
            if (!goal) return false
            if (timeStep.step < goal.condition.end) return true
            // 直接引爆节点
            typeC.get(id).needBombTimes--;
            stateC.get(id).value = DataState.Exploded;
            // 消除的该物品不需要统计收集
            return false
        })

        timeStep.list = newList
    }
}
